#!/usr/bin/env python
# -*- coding:utf-8 -*-

#############################################
# File Name: excel.py
# Mail: 1957875073@qq.com
# Created Time:  2022-4-25 10:17:34
# Description: 有关 excel 的自动化操作
#############################################

from faker import Faker
import pandas as pd
from alive_progress import alive_bar

from utils import pandas_mem


def fake2excel(columns=['name'], rows=1, language='zh_CN', path='./fake2excel.xlsx', ):
    """
    @Author & Date  : CoderWanFeng 2022/5/13 0:12
    @Desc  : columns:list，每列的数据名称，默认是名称
            rows：多少行，默认是1
            language：什么语言，可以填english，默认是中文
            path：输出excel的位置，有默认值
    """
    # 可以选择英语
    if language.lower() == 'english':
        language = 'en_US'
    # 开始造数
    fake = Faker(language)
    excel_dict = {}
    with alive_bar(len(columns) * rows) as bar:
        for column in columns:
            excel_dict[column] = list()
            # excel_dict[column] = map(lambda x: eval('fake.{func}()'.format(func=x)), [column] * rows) # 使用map，会报错
            while len(excel_dict[column]) < rows:
                excel_dict[column].append(eval('fake.{func}()'.format(func=column)))
                bar()
        # 用pandas，将模拟数据，写进excel里面
        writer = pd.ExcelWriter(path)
        data = pd.DataFrame(excel_dict)
        data = pandas_mem.reduce_pandas_mem_usage(data)
        data.to_excel(writer, index=False)
        writer.save()



#!/usr/bin/env python
#-*- coding:utf-8 -*-

#############################################
# File Name: word.py
# Author: 程序员晚枫
# Mail: 1957875073@qq.com
# Created Time:  2022-4-25 10:17:34
# Description: 有关word的自动化操作
#############################################

from win32com.client import constants, gencache
import os  # 目录的操作


def createpdf(wordPath, pdfPath):
    word = gencache.EnsureDispatch('Word.Application')
    doc = word.Documents.Open(wordPath, ReadOnly=1)
    # 转换方法
    doc.ExportAsFixedFormat(pdfPath, constants.wdExportFormatPDF)
    word.Quit()



# 1、文件的批量转换
# 自己指定路径，
# 为了适配wps不能转换doc的问题，这里限定：只能转换docx
def docx2pdf(path, docxSuffix=".docx"):
    wordFiles = []
    # 如果不存在，则不做处理
    if not os.path.exists(path):
        print("path does not exist path = " + path)
        return
    # 判断是否是文件
    elif os.path.isfile(path):
        print("path file type is file " + path)
        wordFiles.append(path)
    # 如果是目录，则遍历目录下面的文件
    elif os.path.isdir(path):
        print(os.listdir(path))
        # 填充路径，补充完整路径
        if not path.endswith("/") or not path.endswith("\\"):
            path = path + "/"
        for file in os.listdir(path):
            if file.endswith(docxSuffix):
                wordFiles.append(path + file)
    print(wordFiles)
    for file in wordFiles:
        filepath = os.path.abspath(file)
        index = filepath.rindex('.')
        pdfPath = filepath[:index] + '.pdf'
        print(pdfPath)
        createpdf(filepath, pdfPath)



# -*- coding: utf-8 -*-
import os
from fpdf import FPDF
from service.pdf import add_watermark_service
import pikepdf
from PyPDF2 import PdfFileReader, PdfFileWriter
from pdf2docx import Converter


#给pdf加水印
def add_watermark():
    pdf_file_in = input("请输入需要添加水印的文件位置：")  # 需要添加水印的文件
    Watermark_Str = input("请输入需要添加的水印内容：")
    print('=' * 20)
    print('正在按要求，给你的PDF文件添加水印，请让程序飞一会儿~')
    print('=' * 20)
    pdf_file_mark = 'watermark.pdf'  # 水印文件
    add_watermark_service.create_watermark(str(Watermark_Str))
    pdf_file_out = '添加了水印的文件.pdf'  # 添加PDF水印后的文件
    add_watermark_service.pdf_add_watermark(pdf_file_in, pdf_file_mark, pdf_file_out)
    print("水印添加结束，请打开电脑上的这个位置，查看结果文件：{path}".format(path=os.getcwd()))

# txt转pdf
def txt2pdf(path,res_pdf='txt2pdf.pdf'):
    pdf = FPDF()
    pdf.add_page()  # Add a page
    pdf.set_font("Arial", size=15)  # set style and size of font
    f = open(path, "r")  # open the text file in read mode
    # insert the texts in pdf
    for x in f:
        pdf.cell(50, 5, txt=x, ln=1, align='C')
    # pdf.output("path where you want to store pdf file\\file_name.pdf")
    pdf.output(res_pdf)


# PDF加密
def encrypt4pdf(path, password, res_pdf='encrypt.pdf'):
    """
    @Author & Date  : CoderWanFeng 2022/5/9 18:27
    @Desc  : path: 存放文件的路径
            password: 你的密码
            res_pdf: 结果文件的名称 ，可以为空，默认是：encrypt.pdf
    """
    pdf = pikepdf.open(path)
    pdf.save(res_pdf, encryption=pikepdf.Encryption(owner=password,  user=password,R=4))
    pdf.close()


# PDF解密
def decrypt4pdf(path, password, res_pdf='decrypt.pdf'):
    pdf = pikepdf.open(path, password=password)
    pdf.save(res_pdf)
    pdf.close()

#合并pdf
def merge2pdf(one_by_one, output):
    """
    @Author & Date  : CoderWanFeng 2022/5/16 23:33
    @Desc  : merge_pdfs(paths=['开篇词.pdf', '中国元宇宙白皮书 (送审稿).pdf'], output='merge.pdf')
    """
    pdf_writer = PdfFileWriter()

    for path in one_by_one:
        pdf_reader = PdfFileReader(path)
        for page in range(pdf_reader.getNumPages()):
            # 把每张PDF页面加入到这个可读取对象中
            pdf_writer.addPage(pdf_reader.getPage(page))

    # 把这个已合并了的PDF文档存储起来
    with open(output, 'wb') as out:
        pdf_writer.write(out)



def pdf2docx(file_path):
    try:
        pdf_name = file_path.split('.')[0]
        word_name = pdf_name + '.docx'
        cv = Converter(file_path)
        cv.convert(word_name)
        cv.close()
    except:
        print('这个文件有问题~！')



#!/usr/bin/env python
# -*- coding:utf-8 -*-

#############################################
# File Name: 文件.py
# Mail: 1957875073@qq.com
# Created Time:  2022-4-25 10:17:34
# Description: 有关 文件 的自动化操作
#############################################


import os
from alive_progress import alive_bar


def replace4filename(path, del_content, replace_content=None):
    """
    :param path: 需要替换的文件夹路径
    :param del_content: 需要删除/替换的内容
    :param replace_content: 替换后的内容，可以不填 = 直接删除
    :return: 
    """
    # 获取该目录下所有文件，存入列表中；不包含子文件夹
    fileList = os.listdir(path)
    work_count = 0
    with alive_bar(len(fileList)) as bar:
        for old_file_name in fileList:  # 依次读取该路径下的文件名
            bar()  # 进度条
            if del_content in old_file_name:

                if replace_content:
                    new_file_name = old_file_name.replace(del_content, replace_content)
                else:
                    new_file_name = old_file_name.replace(del_content, '')
                os.rename(path + os.sep + old_file_name, path + os.sep + new_file_name)
                work_count = work_count + 1
    print("当前目录下，共有{}个文件/文件夹，本次运行共进行了{}个文件/文件夹的重命名".format(len(fileList), work_count))



# import library
import moviepy.editor as mp

#从视频里提取音频
def video2mp3(path, mp3_name=None):
    """
    :param path: 视频文件的路径
    :param mp3_name: mp3的名字，可以为空
    :return:
    """
    # specify the mp4 file here(mention the file path if it is in different directory)
    clip = mp.VideoFileClip(path)
    if mp3_name:
        clip.audio.write_audiofile(str(mp3_name) + '.mp3')
    else:
        # specify the name for mp3 extracted
        clip.audio.write_audiofile('Audio.mp3')
        # you will notice mp3 file will be created at the specified location.



#!/usr/bin/env python
# -*- coding:utf-8 -*-

#############################################
# File Name: ppt.py
# Mail: 1957875073@qq.com
# Created Time:  2022-4-25 10:17:34
# Description: 有关 ppt 的自动化操作
#############################################
import os
from alive_progress import alive_bar
import time

from service.ppt.ppt2pdf_service import ppt2pdf_single


def ppt2pdf(path):
    """
    @Author & Date  : CoderWanFeng 2022/5/9 23:34
    @Desc  : path:存放ppt的路径，必须写绝对路径~！
    """
    # 列出指定目录的内容
    filenames = os.listdir(path)
    # for循环依次访问指定目录的所有文件名
    for filename in filenames:
        # 判断文件的类型，对所有的ppt文件进行处理(ppt文件以ppt或者pptx结尾的)
        if filename.endswith('ppt') or filename.endswith('pptx'):
            # print(filename)           # PPT素材1.pptx -> PPT素材1.pdf
            # 将filename以.进行分割，返回2个信息，文件的名称和文件的后缀名
            base, ext = filename.split('.')  # base=PPT素材1 ext=pdf
            new_name = base + '.pdf'  # PPT素材1.pdf
            # ppt文件的完整位置: C:/Users/Administrator/Desktop/PPT办公自动化/ppt/PPT素材1.pptx
            filename = path + '/' + filename
            # pdf文件的完整位置: C:/Users/Administrator/Desktop/PPT办公自动化/ppt/PPT素材1.pdf
            output_filename = path + '/' + new_name
            # 将ppt转成pdf文件
            ppt2pdf_single(filename, output_filename)
            time.sleep(3)


import numpy as np


def reduce_pandas_mem_usage(df):
    """ iterate through all the columns of a dataframe and modify the data type
        to reduce memory usage.
    """
    # start_mem = df.memory_usage().sum() / 1024 ** 2
    # print('Memory usage of dataframe is {:.2f} MB'.format(start_mem))

    for col in df.columns:
        col_type = df[col].dtype

        if col_type != object:
            c_min = df[col].min()
            c_max = df[col].max()
            if str(col_type)[:3] == 'int':
                if c_min > np.iinfo(np.int8).min and c_max < np.iinfo(np.int8).max:
                    df[col] = df[col].astype(np.int8)
                elif c_min > np.iinfo(np.int16).min and c_max < np.iinfo(np.int16).max:
                    df[col] = df[col].astype(np.int16)
                elif c_min > np.iinfo(np.int32).min and c_max < np.iinfo(np.int32).max:
                    df[col] = df[col].astype(np.int32)
                elif c_min > np.iinfo(np.int64).min and c_max < np.iinfo(np.int64).max:
                    df[col] = df[col].astype(np.int64)
        else:
            if 'date' in col:
                pass
            else:
                df[col] = df[col].astype('category')

    # end_mem = df.memory_usage().sum() / 1024 ** 2
    # print('Memory usage after optimization is: {:.2f} MB'.format(end_mem))
    # print('Decreased by {:.1f}%'.format(100 * (start_mem - end_mem) / start_mem))

    return df



# import the library
from translate import Translator
import qrcode
import string
import random


def transtools(to_lang, content):
    # specifying the language
    translator = Translator(to_lang)
    # typing the message
    translation = translator.translate(content)
    # print the translated message
    print(translation)


def qrcodetools(url):
    # Creating object
    # version: defines size of image from integer(1 to 40), box_size = size of each box in pixels, border = thickness of the border.
    qr = qrcode.QRCode(version=1, box_size=10, border=5)
    # add_date :  pass the input text
    qr.add_data(url)
    # converting into image
    qr.make(fit=True)
    # specify the foreground and background color for the img
    img = qr.make_image(fill='black', back_color='white')
    # store the image
    img.save('qrcode_img.png')

def passwordtools(len = 8):
    """
    @Author & Date  : CoderWanFeng 2022/5/9 11:54
    @Desc  : 随机密码生成器，默认是8位
    @Return  ：
    """
    chars = string.digits + string.ascii_letters
    return ''.join(random.sample(chars * 10, len))



"""
1. 如何设置编辑器字体的大小?
File(文件)-> Settings(设置) -> Editor(编辑器) -> Font(字体), 修改字体的大小
2. 注释: 代码的解释说明
"""

# 1). 导入需要的模块(打开应用程序的模块)
import win32com.client
import os


def ppt2pdf_single(filename, output_filename):
    """
    PPT文件导出为pdf格式
    :param filename: PPT文件的名称
    :param output_filename: 导出的pdf文件的名称
    :return:
    """
    # 2). 打开PPT程序
    ppt_app = win32com.client.Dispatch('PowerPoint.Application')
    # ppt_app.Visible = True  # 程序操作应用程序的过程是否可视化

    # 3). 通过PPT的应用程序打开指定的PPT文件
    # filename = "C:/Users/Administrator/Desktop/PPT办公自动化/ppt/PPT素材1.pptx"
    # output_filename = "C:/Users/Administrator/Desktop/PPT办公自动化/ppt/PPT素材1.pdf"
    ppt = ppt_app.Presentations.Open(filename)

    # 4). 打开的PPT另存为pdf文件。17数字是ppt转图片，32数字是ppt转pdf。
    ppt.SaveAs(output_filename, 32)
    # 退出PPT程序
    ppt_app.Quit()





#!/usr/bin/env python
# -*- coding:utf-8 -*-

#############################################
# File Name: 图片.py
# Mail: 1957875073@qq.com
# Created Time:  2022-4-25 10:17:34
# Description: 有关 图片 的自动化操作
#############################################

import os
# from service.image import add_watermark_service
# 生成词云需要使用的类库
from PIL import Image
from alive_progress import alive_bar




# 自动生成gif
from service.image import add_watermark_service


def image2gif():
    im = Image.open("1.jpg")
    images = []
    images.append(Image.open('2.jpg'))
    images.append(Image.open('3.jpg'))
    im.save('gif.gif', save_all=True, append_images=images, loop=1, duration=1, comment=b"aaabb")

# from wordcloud import WordCloud
# import jieba

# def txt2wordcloud(filename, color="white", result_file="your_wordcloud.png"):
#     """
#     @Author & Date  : CoderWanFeng 2022/4/28 9:26
#     @Desc  : 生成词云的代码，可以添加更多个性化功能
#     @Return  ：
#     """
#     with open(filename, encoding='utf8') as fp:
#         text = fp.read()
#         # 将读取的中文文档进行分词
#         # 接收分词的字符串
#         word_list = jieba.cut(text)
#         # 分词后在单独个体之间加上空格
#         cloud_text = " ".join(word_list)
#
#         # 生成wordcloud对象
#         wc = WordCloud(background_color=color,
#                        max_words=200,
#                        min_font_size=15,
#                        max_font_size=50,
#                        width=400,
#                        font_path="msyh.ttc",  # 默认的简体中文字体，没有会报错
#                        )
#         wc.generate(cloud_text)
#         wc.to_file(result_file)


def add_watermark(file, mark, out="output", color="#8B8B1B", size=30, opacity=0.15, space=75, angle=30):
    """
    @Author & Date  : demo 2022/5/6 14:33
    @Desc  : 给图片添加水印
    @Return  ： 添加了水印的图片，输出到out指定的文件夹
    """
    if os.path.isdir(file):
        names = os.listdir(file)
        with alive_bar(len(names)) as bar:
            for name in names:
                bar()
                image_file = os.path.join(file, name)
                add_watermark_service.add_mark2file(image_file, mark, out, color, size, opacity, space, angle)
    else:
        add_watermark_service.add_mark2file(file, mark, out, color, size, opacity, space, angle)
